UNIT savemef;
(* Copyright 2016-2021 Michael HG Jacobs

   This file is part of 'me'

   me is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the license, or
   (at your option) any later version.

   me is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with 'me'. If not, see <https://www.gnu.org/licenses/>.
   -----------------------------------------------------------------------

   Saving the result of cloning a VDoS from unit clonevdos.pas. This unit
   is used by unit clonevdos.pas
 *)

INTERFACE
USES crt,sysutils,variables,readinput;

PROCEDURE SAVE_RESULT_FILE(OLD_N_EINSTEIN:INTEGER;OLD_STATIC_VOLUME,H_TARGET,
          H_TARGET_CAL,S_TARGET,S_TARGET_CAL,V_TARGET,V_TARGET_CAL,STRETCH_FACTOR:EXTENDED;
          PHASE_DISPERSE:BOOLEAN);
PROCEDURE SAVE_TO_NEW_MEF;
IMPLEMENTATION

TYPE STR14=STRING[14];
     STR4=STRING[4];
     STR2=STRING[2];
VAR I:INTEGER;


PROCEDURE SAVE_HEADER(OLD_N_EINSTEIN:INTEGER;OLD_STATIC_VOLUME,H_TARGET,H_TARGET_CAL,
        S_TARGET,S_TARGET_CAL,V_TARGET,V_TARGET_CAL,STRETCH_FACTOR:EXTENDED;PHASE_DISPERSE:BOOLEAN);
VAR MONTH_NAME:STR14;
    YEAR,DAY,UNIT_PRESSURE:STR4;
    HOUR,MINUTE,SECOND:STR2;
    J:INTEGER;
    FACTOR:EXTENDED;
(* Saves header of the output file with some info results obtained in the clone process *)
BEGIN
  UNIT_PRESSURE:='Pa'; FACTOR:=1;
  IF P_CLONE>1E+06 THEN
  BEGIN
    UNIT_PRESSURE:='MPa'; FACTOR:=1E+06
  END;
  IF P_CLONE>1E+09 THEN
  BEGIN
    UNIT_PRESSURE:='GPa'; FACTOR:=1E+09
  END;
  WRITELN(F_OUTPUT,'# The VDoS in this file is cloned from that in file:');
  (* Check if the input file has a path: if not add it: see READINPUT.PAS *)
  GET_PATH_INPUTFILE(NAME_INPUTFILE);
  WRITELN(F_OUTPUT,'# ',NAME_INPUTFILE);
  (* Results obtained during cloning in unit clonevdos.pas *)
  WRITELN(F_OUTPUT,'#');
  WRITELN(F_OUTPUT,'# Number of frequencies in the VDoS      : ',NUMEIN_CLONE);
  WRITELN(F_OUTPUT,'# Original number of Einstein frequencies: ', OLD_N_EINSTEIN);
  WRITE(F_OUTPUT,'# Clone condition: ',T_CLONE:10:3,' K',' and ',P_CLONE/FACTOR:12:4);
  WRITELN(F_OUTPUT,' ',UNIT_PRESSURE);
  WRITELN(F_OUTPUT,'#');
  WRITELN(F_OUTPUT,'# Old static volume: ',OLD_STATIC_VOLUME*1E6:10:6,' cm3/mol');
  WRITELN(F_OUTPUT,'# New static volume: ',V0_ST*1E6:10:6,' cm3/mol');
  WRITELN(F_OUTPUT,'#');
  WRITELN(F_OUTPUT,'# VDoS stretch factor: ',STRETCH_FACTOR:10:6);
  WRITELN(F_OUTPUT,'#');
  WRITELN(F_OUTPUT,'# At clone condition:  Enthalpy (J/mol)   Entropy (J/K/mol)  Volume (cm3/mol)');
  WRITE(F_OUTPUT,'# Clone database    : ',H_TARGET_CAL:17,' ',S_TARGET_CAL:12:6);
  WRITELN(F_OUTPUT,'        ',V_TARGET_CAL*1E+06:10:6);
  WRITE(F_OUTPUT,'# Original database : ',H_TARGET:17,' ',S_TARGET:12:6);
  WRITELN(F_OUTPUT,'        ',V_TARGET*1E+06:10:6);
  IF (PHASE_DISPERSE=TRUE) AND (NUMEIN_CLONE<10) THEN
  BEGIN
    WRITELN(F_OUTPUT,'#');
    WRITELN(F_OUTPUT,'# Dispersion in Gruneisen, mode-q, anharmonicity parameters may affect thermodynamic');
    WRITELN(F_OUTPUT,'# properties when the number of Einstein frequencies is smaller than 10 !!');
  END;
  WRITELN(F_OUTPUT,'#');
  WRITELN(F_OUTPUT,'# This file calculated by: me',ME_VERSION);
  CURRENT_DATE_TIME(YEAR,MONTH_NAME,DAY,HOUR,MINUTE,SECOND);
  WRITE(F_OUTPUT,'# Constructed on   : ',MONTH_NAME,' ',DAY,', ',YEAR);
  WRITELN(F_OUTPUT,' / ',HOUR,'hr:',MINUTE,'min:',SECOND,'sec');
  FOR J:=1 TO 80 DO WRITE(F_OUTPUT,'#');
  WRITELN(F_OUTPUT);
  FOR J:=1 TO 2 DO WRITELN(F_OUTPUT,'#')
END;

PROCEDURE ADD_BLANKS(VAR F:TEXT;ADD:INTEGER);
(* Add 'add' blanks to file F_OUT *)
VAR J:INTEGER;
BEGIN
  FOR J:=1 TO ADD DO WRITE(F,' ')
END;

PROCEDURE SAVE_TO_NEW_MEF;
VAR STRIN:ANSISTRING;
    ADD,J,LEN:INTEGER;
    SUM:DOUBLE;
(* Saving model parameters of the clone *)
BEGIN
  (* 1. Write name of the compound *)
  WRITE(F_OUTPUT,' ',SUBSTANCE_NAME);
  LEN:=LENGTH(SUBSTANCE_NAME);
  ADD:=12-LEN;
  IF ADD>0 THEN
  BEGIN
    FOR J:=1 TO ADD DO WRITE(F_OUTPUT,' ')
  END;
  ADD_BLANKS(F_OUTPUT,15);
  WRITELN(F_OUTPUT,'(* Name of the substance *)');
  (* 2. Write mass of the substance *)
  WRITE(F_OUTPUT,' ',MOLAR_MASS:11:6); ADD_BLANKS(F_OUTPUT,16);
  WRITELN(F_OUTPUT,'(* Mass of the substance in gr/mol *)');
  (* 3. Number of atoms in the formula unit *)
  WRITE(F_OUTPUT,' ',N_MOL:22); ADD_BLANKS(F_OUTPUT,5);
  WRITELN(F_OUTPUT,'(* Number of atoms in 1 molecular formula unit *)');
  (* 4. Static lattice energy *)
  WRITE(F_OUTPUT,' ',U_REF:22); ADD_BLANKS(F_OUTPUT,5);
  WRITELN(F_OUTPUT,'(* Static lattice energy in J/mol *)');
  (* 5. Volume V0 *)
  WRITE(F_OUTPUT,' ',V0:22); ADD_BLANKS(F_OUTPUT,5);
  WRITELN(F_OUTPUT,'(* Volume/m3/mol at zero K and zero Pa *)');
  (* 6. Static lattice volume *)
  WRITE(F_OUTPUT,' ',V0_ST:22); ADD_BLANKS(F_OUTPUT,5);
  WRITELN(F_OUTPUT,'(* Volume/m3/mol of the static lattice at zero Pa *)');
  (* 7. Name of the Equation of State *)
  WRITE(F_OUTPUT,'  ',EOS_TYPE);
  ADD_BLANKS(F_OUTPUT,25);
  WRITELN(F_OUTPUT,'(* 1=Vinet, 2=Birch-Murnaghan, 3=Keane, 4=Qin EoS of the static lattice *)');
  (* 8. Equation of state of static lattice *)
  WRITE(F_OUTPUT,EOS_ORDER:3); ADD_BLANKS(F_OUTPUT,25);
  CASE EOS_TYPE OF
    1:WRITE(F_OUTPUT,'(* Order of Vinet EoS of the static lattice *)');
    2:WRITE(F_OUTPUT,'(* Order of Birch-Murnaghan EoS of the static lattice *)');
    3:WRITE(F_OUTPUT,'(* Order of Keane EoS of the static lattice *)');
    4:WRITE(F_OUTPUT,'(* Order of Qin et al (2008) EoS of the static lattice *)')
  END;
  WRITELN(F_OUTPUT);
  (* 9. Static bulk modulus *)
  WRITE(F_OUTPUT,' ',K0_ST:22); ADD_BLANKS(F_OUTPUT,5);
  WRITELN(F_OUTPUT,'(* Static lattice bulk modulus/Pa at zero Pa *)');
  (* 10. Pressure derivative bulk modulus: that may be bm1 instead of K0' *)
  WRITE(F_OUTPUT,' ',DK0_ST[1]:22); ADD_BLANKS(F_OUTPUT,5);
  IF (EOS_ORDER<=3) OR (EOS_TYPE=3) THEN
  BEGIN
    WRITELN(F_OUTPUT,'(* Pressure derivative of the static lattice bulk modulus at zero Pa *)')
  END
  ELSE
  BEGIN
    IF EOS_TYPE<>4 THEN 
    BEGIN
      (* Birch-Murnaghan: n-th order *)
      WRITELN(F_OUTPUT,'(* --- 1st pressure derivative coefficient of the static lattice bulk modulus at zero Pa *)')
    END
    ELSE
    BEGIN
      (* Qin et al (2008) *)
      WRITELN(F_OUTPUT,'(* Coefficient "p" in the EoS of Qin et al (2008) *)')
    END
  END;
  IF EOS_TYPE<>3 THEN
  BEGIN
    IF (EOS_ORDER>3) THEN
    BEGIN
      (* Order is ord_eos. Now index J denoted the order *)
      FOR J:=4 TO EOS_ORDER DO
      BEGIN
        WRITE(F_OUTPUT,' ',DK0_ST[J-2]:22); ADD_BLANKS(F_OUTPUT,5);
        IF EOS_TYPE<>4 THEN
        BEGIN
          (* Birch-Murnaghan: n-th order *)
          CASE J OF
            4:WRITELN(F_OUTPUT,'(* --- 2nd pressure derivative coefficient of the static lattice bulk modulus at zero Pa *)');
            5:WRITELN(F_OUTPUT,'(* --- 3rd pressure derivative coefficient of the static lattice bulk modulus at zero Pa *)')
          END;
          IF J>5 THEN
          BEGIN
            WRITELN(F_OUTPUT,'(* --- ',J-2,'th pressure derivative coefficient of the static lattice bulk modulus at zero Pa *)')
          END
        END
        ELSE
        BEGIN
          (* Qin et al (2008) EoS *)
          CASE J OF
            4:WRITELN(F_OUTPUT,'(* Coefficient "q" in the EoS of Qin et al (2008) *)');
            5:WRITELN(F_OUTPUT,'(* Coefficient "m" in the EoS of Qin et al (2008) *)')
          END;
          IF J>5 THEN
          BEGIN
            WRITELN(F_OUTPUT,'(* Coefficient "n" in the EoS of Qin et al (2008) *)')
          END
        END
      END
    END;
  END
  ELSE
  BEGIN
     (* Keane EoS *)
     WRITE(F_OUTPUT,' ',DK0_ST[2]:22); ADD_BLANKS(F_OUTPUT,5);
     WRITELN(F_OUTPUT,'(* Pressure derivative of the static lattice bulk modulus at infinite pressure *)')
  END;
  (* 11. Shear modulus only when elasticity model of Stixrude/Lithgow-Bertelloni,2005
         However program me read shear modulus always when Birch-Murnaghan.
   *)
  IF EOS_TYPE=2 THEN
  BEGIN
    WRITE(F_OUTPUT,' ',G0_ST:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Static lattice shear modulus/Pa at zero Pa *)');
    (* 12. Pressure derivative of shear modulus, it could be gm1 when type code 170 *)
    WRITE(F_OUTPUT,' ',DG0_ST[1]:22); ADD_BLANKS(F_OUTPUT,5);
    IF EOS_ORDER<=3 THEN
      WRITELN(F_OUTPUT,'(* Pressure derivative of the static lattice shear modulus at zero Pa *)')
    ELSE
      WRITELN(F_OUTPUT,'(* --- 1st pressure derivative coefficient of the static lattice shear modulus at zero Pa *)');
    IF EOS_ORDER>3 THEN
    BEGIN
      (* Order is ord_eos. We have freq_e^[3.4.5.6.7,P] for order 4,5,6,7,8 for
         shear modulus derivative 2,3,4,5,6. Now index J denoted the order.
       *)
      FOR J:=4 TO EOS_ORDER DO
      BEGIN
        WRITE(F_OUTPUT,' ',DG0_ST[J-2]:22); ADD_BLANKS(F_OUTPUT,5);
        CASE J OF
          4:WRITELN(F_OUTPUT,'(* --- 2nd pressure derivative coefficient of the static lattice shear modulus at zero Pa *)');
          5:WRITELN(F_OUTPUT,'(* --- 3rd pressure derivative coefficient of the static lattice shear modulus at zero Pa *)')
        END;
        IF J>5 THEN
        BEGIN
          WRITELN(F_OUTPUT,'(* --- ',J-2,'th pressure derivative coefficient of the static lattice shear modulus at zero Pa *)')
        END
      END
    END;
    (* 13. Vibrational parameter for shear modulus *)
    WRITE(F_OUTPUT,' ',NS0:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Vibrational parameter of the shear modulus *)');
  END;
  IF LANDAU_YES>1E-08 THEN
  BEGIN
    (* 14. Flag for Landau parameters: 0=No Landau, 1=2nd order, 2=tricritical
           3=1st order. That is contained in LANDAU_YES
     *)
    WRITE(F_OUTPUT,'  ',LANDAU_YES); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0=No Landau contribution, 1=2nd order, 2=Tricritical, 3=1st order *)');
    (* For 2nd order and tricritical write: a,CS,Tc0,3rd-law.
       For 1st order write: a,CS,Tr0,Q0,3rd-law
     *)
    WRITE(F_OUTPUT,' ',LANDAU[1]:22); ADD_BLANKS(F_OUTPUT,5);
    WRITE(F_OUTPUT,'(* Landau parameter: ');
    WRITELN(F_OUTPUT,'a *)');
    (* 16. Clapeyron slope of the critical line *)
    WRITE(F_OUTPUT,' ',LANDAU[2]:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Landau Clapeyron slope Pa/K of the critical phase boundary *)');
    (* 17. Critical temperature at zero pressure *)
    WRITE(F_OUTPUT,' ',LANDAU[3]:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Landau critical temperature at zero pressure *)');
    (* 18. In case of 1st order: the order parameter Q0 which is the same for
           at all transition temperatures
     *)
    IF ABS(LANDAU_YES-3)<1E-08 THEN
    BEGIN
      WRITE(F_OUTPUT,' ',LANDAU[4]:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Order parameter, Q0, at the transition *)')
    END;
    (* 19. Entropy correction: 0=unnecessary 1=necessary *)
    IF ABS(LANDAU_YES-3)<1E-08 THEN WRITE(F_OUTPUT,' ',LANDAU[5]:22)
    ELSE WRITE(F_OUTPUT,' ',LANDAU[4]:22);
    ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Landau 3rd law entropy correction: 0=unnecessary, 1=necessary *)')
  END
  ELSE
  BEGIN
    (* No Landau contribution present *)
    WRITE(F_OUTPUT,'  0'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0=No Landau contribution, 1=2nd order, 2=Tricritical, 3=1st order *)');
  END;
  (* Magnetism *)
  IF MAGNETIC_YES>0 THEN
  BEGIN
    (* 20. Magnetic contributions present *)
    WRITE(F_OUTPUT,'  1'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0=No magnetic contribution, 1=magnetic contribution *)');
    (* 21. Number of magnetic terms in the MacLaurin series. The summation goes from zero
           to maximum 50. 
     *)
    WRITE(F_OUTPUT,' ',NUM_MCLAURIN_TERMS:2); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* Magnetic: number of terms in the MacLaurin series expansion >2 *)');
    (* 22. Fraction of energy above critical point required to convert from the
           ferromagnetic to the paramagnetic state
     *)
    WRITE(F_OUTPUT,' ',INDEN_P:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Magnetic: Ratio Energy(tau>1)/total Energy for magnetic transition with tau=T/Tc *)');
    (* 23. Critical temperature *)
    WRITE(F_OUTPUT,' ',T_CURIE:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Magnetic: Critical temperature Tc *)');
    (* 24. Magnetic moment Bohr-magnetons  *)
    WRITE(F_OUTPUT,' ',MAG_MOMENT:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Magnetic: magnetic moment in Bohr magneton *)');
    (* 25. Exponent Cv expression below critical temperature *)
    WRITE(F_OUTPUT,' ',INDEN_M:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Magnetic: exponent (m) in Inden''s (1981) expression for Cv below Tc *)');
    (* 26. Exponent Cv expression above critical temperature *)
    WRITE(F_OUTPUT,' ',INDEN_N:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Magnetic: exponent (n) in Inden''s (1981) expression for Cv above Tc *)');
    (* 27. Number of magnetic atoms in one formula unit *)
    WRITE(F_OUTPUT,' ',NUM_MAG_ATOMS:22); ADD_BLANKS(F_OUTPUT,5);
    WRITELN(F_OUTPUT,'(* Magnetic: number of magnetic atoms per formula unit *)')
  END
  ELSE
  BEGIN
    (* No magnetic contributions present *)
    WRITE(F_OUTPUT,'  0'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0=No magnetic contribution, 1=magnetic contribution *)');
  END;
  (* Electronic contributions *)
  IF ELECTRONIC_YES=0 THEN
  BEGIN
    WRITE(F_OUTPUT,'  0'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* No electronic contribution *)')
  END
  ELSE
  BEGIN
    IF ELECTRONIC_YES=1 THEN
    BEGIN
      WRITE(F_OUTPUT,'  1'); ADD_BLANKS(F_OUTPUT,25);
      WRITELN(F_OUTPUT,'(* Free electron gas model *)');
      WRITELN(F_OUTPUT,'# Cv=a*T*(V/V0)^gamma_el');
      (* 29a. Free electron gas: electronic Gruneisen *)
      WRITE(F_OUTPUT,' ',GAMMA_EL:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Electronic Gruneisen parameter, gamma_el *)');
      WRITE(F_OUTPUT,' ',C_EL[1]:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Electronic coefficient, a, in J/mol/K/K *)')
    END;
    IF ELECTRONIC_YES=2 THEN
    BEGIN
      (* 29b. Extended free-electron gas model to take electron-phonon into account *)
      WRITE(F_OUTPUT,'  2'); ADD_BLANKS(F_OUTPUT,25);
      WRITELN(F_OUTPUT,'(* Extended free electron gas model: electron-phonon coupling *)');
      WRITELN(F_OUTPUT,'# Cv=(a1*T*exp(-b1*T)+a2*T^2*exp(-b2*T)+a3*T^3*exp(-b3*T)+c1*T+c2*(1-1/(c3*T+1)))*(V/V0)^gamma_el');
      (* 29b. Extended free electron gas: electronic Gruneisen *)
      WRITE(F_OUTPUT,' ',GAMMA_EL:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Electronic Gruneisen parameter, gamma_el *)');
      WRITELN(F_OUTPUT,'#   a               b              c');
      WRITELN(F_OUTPUT,'# ----------------------------------------------');
      WRITELN(F_OUTPUT,'  1',' ',A_EL[1]:14,' ',B_EL[1]:14,' ',C_EL[1]:14);
      WRITELN(F_OUTPUT,'  2',' ',A_EL[2]:14,' ',B_EL[2]:14,' ',C_EL[2]:14);
      WRITELN(F_OUTPUT,'  3',' ',A_EL[3]:14,' ',B_EL[3]:14,' ',C_EL[3]:14)
    END;
    IF ELECTRONIC_YES=3 THEN
    BEGIN
      WRITE(F_OUTPUT,'  3'); ADD_BLANKS(F_OUTPUT,25);
      WRITELN(F_OUTPUT,'(* Crystal field electronic contribution *)');
      (* 30. Crystal Field energies and degeneracies *)
      WRITE(F_OUTPUT,' ',NUM_CF_SITES:2); ADD_BLANKS(F_OUTPUT,25);
      WRITELN(F_OUTPUT,'(* Number of sites M1, M2, M3: maximum 3 sites *)');
      IF NUM_CF_SITES>0 THEN
      BEGIN
        WRITE(F_OUTPUT,' ',M1_LEVELS:2); ADD_BLANKS(F_OUTPUT,25);
        WRITELN(F_OUTPUT,'(* Number of Crystal field energy levels of the M1 site *)')
      END;
      (* 31. Number of energy levels M2 site *)
      IF NUM_CF_SITES>1 THEN
      BEGIN
        WRITE(F_OUTPUT,' ',M2_LEVELS:2); ADD_BLANKS(F_OUTPUT,25);
        WRITELN(F_OUTPUT,'(* Number of Crystal field energy levels of the M2 site *)')
      END;
      (* 32. Number of energy levels M3 site *)
      IF NUM_CF_SITES>2 THEN
      BEGIN
        WRITE(F_OUTPUT,' ',M3_LEVELS:2); ADD_BLANKS(F_OUTPUT,25);
        WRITELN(F_OUTPUT,'(* Number Crystal field energy levels of the M3 site *)')
      END;
      (* 33. Number of atoms per formula unit having crystal-field splitting *)
      WRITE(F_OUTPUT,' ',NUM_CF_ATOMS:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Number of atoms per formula unit having crystal-field splitting *)');
      (* Magnetic correction factor *)
      WRITE(F_OUTPUT,' ',MAGNETIC_CORR_FACTOR:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Magnetic correction factor (mf): Helmholtz A=na.RT.g(t).mf.ln(1+b) *)');
      WRITELN(F_OUTPUT,'# Energy/cm^-1    Degeneracy Gruneisen parameter');
      WRITELN(F_OUTPUT,'# ----------------------------------------------');
      FOR J:=1 TO M1_LEVELS+M2_LEVELS+M3_LEVELS DO
      BEGIN
        WRITE(F_OUTPUT,' ',E_CF[J]:14,' ',ROUND(G_CF[J]):3);
        ADD_BLANKS(F_OUTPUT,9); WRITELN(F_OUTPUT,GAMMA_CF[J]:14)
      END;
      (* Crystal-field extended with a free-electron contribution *)
      WRITELN(F_OUTPUT,'# Extra Free electron gas contribution');
      WRITELN(F_OUTPUT,'# Cv(electronic)=Cv(crystal-Field)+a*T*(V/V0)^gamma_el');
      (* 34. Extra Free electron gas added to crystal-Field: electronic Gruneisen=2/3 *)
      WRITE(F_OUTPUT,' ',GAMMA_EL:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Electronic Gruneisen parameter, gamma_el *)');
      (* 35. Extra Free electron gas added to crystal-field: The electronic coefficient
       *)
      WRITE(F_OUTPUT,' ',C_EL[1]:22); ADD_BLANKS(F_OUTPUT,5);
      WRITELN(F_OUTPUT,'(* Electronic coefficient, a, in J/K/K/mol atoms that give crystal-field splitting *)')
    END
  END;
  (* Cation disorder *)
  IF CATION_DISORDER_YES>0 THEN
  BEGIN
    WRITE(F_OUTPUT,'  1'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0=No cation disorder, 1=cation disorder: model of O''Neill and Navrotsky (1983,1984) *)');
    WRITE(F_OUTPUT,'#'); ADD_BLANKS(F_OUTPUT,27);
    WRITELN(F_OUTPUT,'(* Hint=(h1+h2y)y, Sint=(s1+s2y)y: y=sitefraction metallic ion *)');
    FOR J:=1 TO 6 DO
    BEGIN
      WRITE(F_OUTPUT,' ',CATION_DO[J]:22); ADD_BLANKS(F_OUTPUT,5);
      CASE J OF
        1:WRITELN(F_OUTPUT,'(* 1st Coefficient interchange enthalpy in J/mol *)');
        2:WRITELN(F_OUTPUT,'(* 2st Coefficient interchange enthalpy in J/mol *)');
        3:WRITELN(F_OUTPUT,'(* 1st Coefficient interchange entropy in J/K/mol *)');
        4:WRITELN(F_OUTPUT,'(* 2st Coefficient interchange entropy in J/K/mol *)');
        5:WRITELN(F_OUTPUT,'(* # moles of tetrahedral sites per molecular formula unit *)');
        6:WRITELN(F_OUTPUT,'(* # moles of octahedral sites per molecular formula unit *)')
      END
    END
  END
  ELSE
  BEGIN
    WRITE(F_OUTPUT,'  0'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0=No cation disorder, 1=cation disorder: model of O''Neill and Navrotsky (1983,1984) *)');
  END;
  (* Defects: monovacancies. Oganov-vib.doc, Dorogokupets & Oganov (2007),
     Phys Rev B 75:024115
   *)
  IF DEFECT_YES>0 THEN
  BEGIN
    WRITE(F_OUTPUT,'  1'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0= No monovacancy defects, 1=monovacancy defects *)');
    WRITE(F_OUTPUT,'#'); ADD_BLANKS(F_OUTPUT,27);
    WRITELN(F_OUTPUT,'(* Model: Dorogokupets & Oganov 2007 Phys Rev B 75:024115 *)');
    WRITE(F_OUTPUT,'#'); ADD_BLANKS(F_OUTPUT,27);
    WRITELN(F_OUTPUT,'(* Helmholtz energy = -(3/2)nRT*exp(s*eta^f-h*eta^g/T) *)');
    WRITE(F_OUTPUT,'#'); ADD_BLANKS(F_OUTPUT,27);
    WRITELN(F_OUTPUT,'(* with n=# atoms/formula unit, eta=V/V0 *)');
    FOR J:=1 TO 4 DO
    BEGIN
      WRITE(F_OUTPUT,' ',DEFECT[J]:22); ADD_BLANKS(F_OUTPUT,5);
      CASE J OF
        1:WRITELN(F_OUTPUT,'(* h: enthalpy of formation, K *)');
        2:WRITELN(F_OUTPUT,'(* s: entropy, no unit *)');
        3:WRITELN(F_OUTPUT,'(* f *)');
        4:WRITELN(F_OUTPUT,'(* g *)');
      END
    END
  END
  ELSE
  BEGIN
    WRITE(F_OUTPUT,'  0'); ADD_BLANKS(F_OUTPUT,25);
    WRITELN(F_OUTPUT,'(* 0=No monovacancy defects, 1=monovacancy defects: Dorogokupets & Oganov 2007 Phys Rev B 75:024115 *)');
  END;
  (* Vibrational parameters *)
  WRITE(F_OUTPUT,'# Anharmonicity is described with perturbation theory of Oganov &');
  WRITELN(F_OUTPUT,' Dorogokupets (2004): a=a0(V/V0)^z');
  (* Number of Einstein modes *)
  WRITE(F_OUTPUT,'  ',N_EINSTEIN);
  STR(N_EINSTEIN,STRIN); LEN:=LENGTH(STRIN);
  ADD:=27-LEN; ADD_BLANKS(F_OUTPUT,ADD-1);
  WRITELN(F_OUTPUT,'(* Number of Einstein modes *)');
  IF GRUNEISEN_TYPE=2 THEN WRITE(F_OUTPUT,'  2') ELSE WRITE(F_OUTPUT,'  1');
  ADD_BLANKS(F_OUTPUT,25);
  WRITE(F_OUTPUT,'(* 1=Gruneisen expression by Al''tshuler et al, 1987');
  WRITELN(F_OUTPUT,' and 2=that by Stixrude & Lithgow-Bertelloni, 2005 *)');
  WRITELN(F_OUTPUT,'#  j   Theta/K     Fraction          Gamma      Mode_m(q)   Gamma_inf  a0             z');
  WRITE(F_OUTPUT,'# ');
  FOR J:=1 TO 93 DO WRITE(F_OUTPUT,'-');
  WRITELN(F_OUTPUT);
  (* Values of vibrational parameters: skip first three Kieffer AC modes *)
  FOR J:=1 TO N_EINSTEIN DO
  BEGIN
    WRITE(F_OUTPUT,J:4,' ',THETA0[J]:12:6,' ',FRAC[J]:17,' ');
    WRITE(F_OUTPUT,GAMMA0[J]:10:7,' ',Q0[J]:10:7,' ');
    WRITE(F_OUTPUT,' ',GAMMA_INF[J]:10:7,' ',ANH_A0[J]:14,' ');
    WRITELN(F_OUTPUT,ANH_Z0[J]:10:7)
  END;
  (* Determine the sum of the fractions and multiply with 100 *)
  SUM:=0;
  FOR J:=1 TO N_EINSTEIN DO SUM:=SUM+FRAC[J];
  WRITELN(F_OUTPUT,'#');
  WRITELN(F_OUTPUT,'# sum of all fractions: ',SUM:17);
  FOR J:=1 TO 95 DO WRITE(F_OUTPUT,'#');
  (* Write references *)
  WRITELN(F_OUTPUT,'#');
  WRITELN(F_OUTPUT,'# References');
  WRITELN(F_OUTPUT,'# ----------');
  WRITELN(F_OUTPUT,'# Al''tshuler LV, Brusnikin SE, Kuz''menkov EA (1987) Isotherms and Gruneisen functions for 25');
  WRITELN(F_OUTPUT,'#    metals. J Appl Mech Tech Phys 28:129-141');
  WRITELN(F_OUTPUT,'# Birch F (1952) Elasticity and constitution of the Earth''s interior. J Geophys Res 57:227-286');
  WRITELN(F_OUTPUT,'# Jacobs MHG, Schmid-Fetzer R, van den Berg AP (2013) An alternative use of Kieffer''s lattice');
  WRITELN(F_OUTPUT,'#    dynamics model using vibrational density of states for constructing thermodynamic');
  WRITELN(F_OUTPUT,'#    databases. Phys Chem Minerals 40:207-227');
  WRITELN(F_OUTPUT,'# Keane A (1954) An investigation of finite strain in an isotropic material subjected to');
  WRITELN(F_OUTPUT,'#    hydrostatic pressure and its seismological applications. Aust J Phys 7:322-333');
  WRITELN(F_OUTPUT,'# Oganov AR, Dorogokupets P (2004) Intrinsic anharmonicity in equations of state and');
  WRITELN(F_OUTPUT,'#    thermodynamics of solids. J Phys Condens Matter 16:1351-1360');
  WRITELN(F_OUTPUT,'# O''Neil HStC, Navrotsky A (1983) Simple spinels: crystallographic parameters, cation radii,');
  WRITELN(F_OUTPUT,'#    lattice energies, and cation distribution. Am Mineral 68:181-194');
  WRITELN(F_OUTPUT,'# O''Neil HStC, Navrotsky A (1984) Cation distributions and thermodynamic properties of');
  WRITELN(F_OUTPUT,'#    binary spinel solid solutions. Am Mineral 69:733-753');
  WRITELN(F_OUTPUT,'# Qin T, Drautz R, Pettifor DG (2008) Equation of State for transition metals. Phys Rev B');
  WRITELN(F_OUTPUT,'#    77:229103(R)');
  WRITELN(F_OUTPUT,'# Stixrude L, Lithgow-Bertelloni (2005) Thermodynamics of mantle minerals - I. Physical');
  WRITELN(F_OUTPUT,'#    properties. Geophys J Int 162:610-632');
  WRITELN(F_OUTPUT,'# Vinet P, Ferrante J, Rose JH, Smith JR (1989) Universal features of the equation of state of');
  WRITELN(F_OUTPUT,'#    solids. J Phys Condens Matter 1:1941-1963');
  FOR I:=1 TO 95 DO WRITE(F_OUTPUT,'#');
  (* Write examples *)
  WRITELN(F_OUTPUT);
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Example 1: making calculations at 1 bar pressure (program ''me'')');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Isobaric calculation');
  WRITELN(F_OUTPUT,' 1                          (* 1=Isobaric, 2=Isothermal, 3=VDoS, 4=Isentropic, 5=Hugoniot *)');
  WRITELN(F_OUTPUT,' 10    2000   30            (* Temperature range and step: T_start/K, T_end/K, T_step/K *)');
  WRITELN(F_OUTPUT,' 1e+05 1e+05  0.0           (* Pressure range and step   : P_start/Pa, P_end/Pa, P_step/Pa *)');
  WRITELN(F_OUTPUT,' 1                          (* Output: 1=Output to screen and file, 0=Output to file only *)');
  WRITELN(F_OUTPUT,'# Output file name');
  WRITELN(F_OUTPUT,' /home/michel/geodata/mgo/mgo.out');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Example 2: making calculations at 300 K (program ''me'')');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Isothermal calculation');
  WRITELN(F_OUTPUT,'# 2                          (* 1=Isobaric, 2=Isothermal, 3=VDoS, 4=Isentropic, 5=Hugoniot *)');
  WRITELN(F_OUTPUT,'# 300    300    0.0          (* Temperature range and step: T_start/K, T_end/K, T_step/K *)');
  WRITELN(F_OUTPUT,'# 1e+05 1e+11  1e+09         (* Pressure range and step   : P_start/Pa, P_end/Pa, P_step/Pa *)');
  WRITELN(F_OUTPUT,'# 1                          (* Output: 1=Output to screen and file, 0=Output to file only *)');
  WRITELN(F_OUTPUT,'# Output file name');
  WRITELN(F_OUTPUT,'# /home/michel/geodata/mgo/mgo.out');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Example 3: making a VDoS at 1 bar pressure and 30 Kelvin (program ''me'')');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# VDoS calculation');
  WRITELN(F_OUTPUT,'# 3                          (* 1=Isobaric, 2=Isothermal, 3=VDoS, 4=Isentropic, 5=Hugoniot *)');
  WRITELN(F_OUTPUT,'# 1                          (* Desired unit: 1=cm^-1, 2=THz, 3=meV *)');
  WRITELN(F_OUTPUT,'# 1e+05  30                  (* Pressure/Pa, Temperature/K at which VDoS is calculated *)');
  WRITELN(F_OUTPUT,'# Script file for GnuPlot:');
  WRITELN(F_OUTPUT,'# /home/geodata/mgo/mgo.grf');
  WRITELN(F_OUTPUT,'# 1                          (* 0=No experimental/abinitio VDoS, 1=experimental/abinitio VDoS *)');
  WRITELN(F_OUTPUT,'# When the preceding line defines an experimental/abinitio VDoS add the following 3 lines:');
  WRITELN(F_OUTPUT,'# File with experimental/abinitio VDoS (X,Y) points:');
  WRITELN(F_OUTPUT,'# /home/michel/geodata/mgo/vdosmgo.exp');
  WRITELN(F_OUTPUT,'# 1                           (* Unit experimental/abinitio VDoS: 1=cm^-1, 2=THz, 3=meV *)');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Example 4: making a VDoS from experimental/abinitio VDoS (program ''medos'')');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# 60                        (* Desired number of Einstein temperatures *)');
  WRITELN(F_OUTPUT,'# Output file serving as input file for program ''me''');
  WRITELN(F_OUTPUT,'# /home/geodata/mgo/mgo60.mef');
  WRITELN(F_OUTPUT,'# 1                         (* VDoS unit: 1=cm^-1, 2=THz, 3=meV *)');
  WRITELN(F_OUTPUT,'# File containing experimental/abinitio VDoS (X,Y) points');
  WRITELN(F_OUTPUT,'# /home/michel/geodata/mgo/vdosmgo.exp');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Example 5: Calculating along an isentropic path (program ''me'')');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Isentropic calculation');
  WRITELN(F_OUTPUT,'# 4                          (* 1=Isobaric, 2=Isothermal, 3=VDoS, 4=Isentropic, 5=Hugoniot *)');
  WRITELN(F_OUTPUT,'# 103.633                    (* Constant entropy, J/K/mol *)');
  WRITELN(F_OUTPUT,'# 1e+05 1e+11  5e+09         (* Pressure range and step   : P_start/Pa, P_end/Pa, P_step/Pa *)');
  WRITELN(F_OUTPUT,'# 1                          (* Output: 1=Output to screen and file, 0=Output to file only *)');
  WRITELN(F_OUTPUT,'# Output file name');
  WRITELN(F_OUTPUT,'# /home/michel/geodata/mgo/mgo.out');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Example 6: Calculating along a Hugoniot (program ''me'')');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Hugoniot calculation');
  WRITELN(F_OUTPUT,'# 5                          (* 1=Isobaric, 2=Isothermal, 3=VDoS, 4=Isentropic, 5=Hugoniot *)');
  WRITELN(F_OUTPUT,'# 300                        (* Temperature in Kelvin of the sample at 1 bar *)');
  WRITELN(F_OUTPUT,'# 1e+05 1e+11  5e+09         (* Pressure range and step   : P_start/Pa, P_end/Pa, P_step/Pa *)');
  WRITELN(F_OUTPUT,'# 1                          (* Output: 1=Output to screen and file, 0=Output to file only *)');
  WRITELN(F_OUTPUT,'# Output file name');
  WRITELN(F_OUTPUT,'# /home/michel/geodata/mgo/mgo.out');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'# Example 7: Clone a VDoS (program ''me'')');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  WRITELN(F_OUTPUT,'## Cloning a VDoS to a VDoS with a smaller number of Einstein frequencies');
  WRITELN(F_OUTPUT,'# 6                          (* 1=Isobaric, 2=Isothermal, 3=VDoS, 4=Isentropic, 5=Hugoniot, 6=Clone *)');
  WRITELN(F_OUTPUT,'# 10                         (* Desired number of Einstein frequencies in the VDoS *)');
  WRITELN(F_OUTPUT,'# 1e+05 1173                 (* Clone condition P/Pa, T/K at which enthalpy and entropy must remain unchanged *)');
  WRITELN(F_OUTPUT,'## Output file name');
  WRITELN(F_OUTPUT,'# /home/michel/geodata/mgo/mgo.out');
  WRITELN(F_OUTPUT,'# -------------------------------------------------------------------------------------------');
  FOR I:=1 TO 95 DO WRITE(F_OUTPUT,'#'); 
END;

PROCEDURE SAVE_RESULT_FILE(OLD_N_EINSTEIN:INTEGER;OLD_STATIC_VOLUME,H_TARGET,
          H_TARGET_CAL,S_TARGET,S_TARGET_CAL,V_TARGET,V_TARGET_CAL,STRETCH_FACTOR:EXTENDED;
          PHASE_DISPERSE:BOOLEAN);
(* V_TARGET is the volume at T_clone, P_clone *)
BEGIN
  (* F_OUTPUT:TEXT is a global variable defined in variables.pas *)
  SET_WRITE_FILE(F_OUTPUT,NAME_OUTPUTFILE);
  SAVE_HEADER(OLD_N_EINSTEIN,OLD_STATIC_VOLUME,H_TARGET,H_TARGET_CAL,S_TARGET,S_TARGET_CAL,
              V_TARGET,V_TARGET_CAL,STRETCH_FACTOR,PHASE_DISPERSE);
  SAVE_TO_NEW_MEF;
  CLOSE(F_OUTPUT);
  WRITELN('Normal termination...')
END;

END.
